/*
Copyright (c) 2021, Kazuhisa Yokota, JN1DFF
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
* Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution.
* Neither the name of the <organization> nor the names of its contributors 
  may be used to endorse or promote products derived from this software 
  without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
    sine wave table for Bell202 FSK signal
*/

#define SYS_CLK_KHZ 132000
#define BIT_CYCLE 440
#define MARK_CYCLE 440
#define SPACE_CYCLE 240
#define MARK_TAB_LEN 807
#define SPACE_TAB_LEN 640
#define PHASE_CYCLE 6
#define PWM_CYCLE 250

static const uint16_t mark_tab[] = {
0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xf9, 0xf9, 0xf9, 0xf9, 0xf8, 0xf8, 0xf8, 0xf8, 0xf7, 
0xf7, 0xf6, 0xf6, 0xf5, 0xf5, 0xf4, 0xf4, 0xf3, 0xf3, 0xf2, 0xf1, 0xf1, 0xf0, 0xef, 0xef, 0xee, 
0xed, 0xec, 0xec, 0xeb, 0xea, 0xe9, 0xe8, 0xe7, 0xe6, 0xe5, 0xe4, 0xe3, 0xe2, 0xe1, 0xe0, 0xdf, 
0xde, 0xdd, 0xdb, 0xda, 0xd9, 0xd8, 0xd7, 0xd5, 0xd4, 0xd3, 0xd2, 0xd0, 0xcf, 0xce, 0xcc, 0xcb, 
0xc9, 0xc8, 0xc6, 0xc5, 0xc4, 0xc2, 0xc1, 0xbf, 0xbe, 0xbc, 0xba, 0xb9, 0xb7, 0xb6, 0xb4, 0xb3, 
0xb1, 0xaf, 0xae, 0xac, 0xaa, 0xa9, 0xa7, 0xa5, 0xa4, 0xa2, 0xa0, 0x9f, 0x9d, 0x9b, 0x99, 0x98, 
0x96, 0x94, 0x92, 0x91, 0x8f, 0x8d, 0x8b, 0x89, 0x88, 0x86, 0x84, 0x82, 0x81, 0x7f, 0x7d, 0x7b, 
0x79, 0x78, 0x76, 0x74, 0x72, 0x71, 0x6f, 0x6d, 0x6b, 0x69, 0x68, 0x66, 0x64, 0x62, 0x61, 0x5f, 
0x5d, 0x5b, 0x5a, 0x58, 0x56, 0x55, 0x53, 0x51, 0x50, 0x4e, 0x4c, 0x4b, 0x49, 0x47, 0x46, 0x44, 
0x43, 0x41, 0x40, 0x3e, 0x3c, 0x3b, 0x39, 0x38, 0x36, 0x35, 0x34, 0x32, 0x31, 0x2f, 0x2e, 0x2c, 
0x2b, 0x2a, 0x28, 0x27, 0x26, 0x25, 0x23, 0x22, 0x21, 0x20, 0x1f, 0x1d, 0x1c, 0x1b, 0x1a, 0x19, 
0x18, 0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 0x10, 0x0f, 0x0e, 0x0e, 0x0d, 0x0c, 0x0b, 0x0b, 
0x0a, 0x09, 0x09, 0x08, 0x07, 0x07, 0x06, 0x06, 0x05, 0x05, 0x04, 0x04, 0x03, 0x03, 0x02, 0x02, 
0x02, 0x02, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x04, 0x04, 0x05, 
0x05, 0x06, 0x06, 0x07, 0x07, 0x08, 0x09, 0x09, 0x0a, 0x0b, 0x0b, 0x0c, 0x0d, 0x0e, 0x0e, 0x0f, 
0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1f, 0x20, 
0x21, 0x22, 0x23, 0x25, 0x26, 0x27, 0x28, 0x2a, 0x2b, 0x2c, 0x2e, 0x2f, 0x31, 0x32, 0x34, 0x35, 
0x36, 0x38, 0x39, 0x3b, 0x3c, 0x3e, 0x40, 0x41, 0x43, 0x44, 0x46, 0x47, 0x49, 0x4b, 0x4c, 0x4e, 
0x50, 0x51, 0x53, 0x55, 0x56, 0x58, 0x5a, 0x5b, 0x5d, 0x5f, 0x61, 0x62, 0x64, 0x66, 0x68, 0x69, 
0x6b, 0x6d, 0x6f, 0x71, 0x72, 0x74, 0x76, 0x78, 0x79, 0x7b, 0x7d, 0x7f, 0x81, 0x82, 0x84, 0x86, 
0x88, 0x89, 0x8b, 0x8d, 0x8f, 0x91, 0x92, 0x94, 0x96, 0x98, 0x99, 0x9b, 0x9d, 0x9f, 0xa0, 0xa2, 
0xa4, 0xa5, 0xa7, 0xa9, 0xaa, 0xac, 0xae, 0xaf, 0xb1, 0xb3, 0xb4, 0xb6, 0xb7, 0xb9, 0xba, 0xbc, 
0xbe, 0xbf, 0xc1, 0xc2, 0xc4, 0xc5, 0xc6, 0xc8, 0xc9, 0xcb, 0xcc, 0xce, 0xcf, 0xd0, 0xd2, 0xd3, 
0xd4, 0xd5, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 
0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xec, 0xed, 0xee, 0xef, 0xef, 0xf0, 0xf1, 0xf1, 0xf2, 
0xf3, 0xf3, 0xf4, 0xf4, 0xf5, 0xf5, 0xf6, 0xf6, 0xf7, 0xf7, 0xf8, 0xf8, 0xf8, 0xf8, 0xf9, 0xf9, 
0xf9, 0xf9, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xfa, 0xf9, 
0xf9, 0xf9, 0xf9, 0xf8, 0xf8, 0xf8, 0xf8, 0xf7, 0xf7, 0xf6, 0xf6, 0xf5, 0xf5, 0xf4, 0xf4, 0xf3, 
0xf3, 0xf2, 0xf1, 0xf1, 0xf0, 0xef, 0xef, 0xee, 0xed, 0xec, 0xec, 0xeb, 0xea, 0xe9, 0xe8, 0xe7, 
0xe6, 0xe5, 0xe4, 0xe3, 0xe2, 0xe1, 0xe0, 0xdf, 0xde, 0xdd, 0xdb, 0xda, 0xd9, 0xd8, 0xd7, 0xd5, 
0xd4, 0xd3, 0xd2, 0xd0, 0xcf, 0xce, 0xcc, 0xcb, 0xc9, 0xc8, 0xc6, 0xc5, 0xc4, 0xc2, 0xc1, 0xbf, 
0xbe, 0xbc, 0xba, 0xb9, 0xb7, 0xb6, 0xb4, 0xb3, 0xb1, 0xaf, 0xae, 0xac, 0xaa, 0xa9, 0xa7, 0xa5, 
0xa4, 0xa2, 0xa0, 0x9f, 0x9d, 0x9b, 0x99, 0x98, 0x96, 0x94, 0x92, 0x91, 0x8f, 0x8d, 0x8b, 0x89, 
0x88, 0x86, 0x84, 0x82, 0x81, 0x7f, 0x7d, 0x7b, 0x79, 0x78, 0x76, 0x74, 0x72, 0x71, 0x6f, 0x6d, 
0x6b, 0x69, 0x68, 0x66, 0x64, 0x62, 0x61, 0x5f, 0x5d, 0x5b, 0x5a, 0x58, 0x56, 0x55, 0x53, 0x51, 
0x50, 0x4e, 0x4c, 0x4b, 0x49, 0x47, 0x46, 0x44, 0x43, 0x41, 0x40, 0x3e, 0x3c, 0x3b, 0x39, 0x38, 
0x36, 0x35, 0x34, 0x32, 0x31, 0x2f, 0x2e, 0x2c, 0x2b, 0x2a, 0x28, 0x27, 0x26, 0x25, 0x23, 0x22, 
0x21, 0x20, 0x1f, 0x1d, 0x1c, 0x1b, 0x1a, 0x19, 0x18, 0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 
0x10, 0x0f, 0x0e, 0x0e, 0x0d, 0x0c, 0x0b, 0x0b, 0x0a, 0x09, 0x09, 0x08, 0x07, 0x07, 0x06, 0x06, 
0x05, 0x05, 0x04, 0x04, 0x03, 0x03, 0x02, 0x02, 0x02, 0x02, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x02, 
0x02, 0x02, 0x02, 0x03, 0x03, 0x04, 0x04, 0x05, 0x05, 0x06, 0x06, 0x07, 0x07, 0x08, 0x09, 0x09, 
0x0a, 0x0b, 0x0b, 0x0c, 0x0d, 0x0e, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 
0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1f, 0x20, 0x21, 0x22, 0x23, 0x25, 0x26, 0x27, 0x28, 0x2a, 
0x2b, 0x2c, 0x2e, 0x2f, 0x31, 0x32, 0x34, 0x35, 0x36, 0x38, 0x39, 0x3b, 0x3c, 0x3e, 0x40, 0x41, 
0x43, 0x44, 0x46, 0x47, 0x49, 0x4b, 0x4c, 0x4e, 0x50, 0x51, 0x53, 0x55, 0x56, 0x58, 0x5a, 0x5b, 
0x5d, 0x5f, 0x61, 0x62, 0x64, 0x66, 0x68, 0x69, 0x6b, 0x6d, 0x6f, 0x71, 0x72, 0x74, 0x76, 0x78, 
0x79, 0x7b, 0x7d, 0x7f, 0x81, 0x82, 0x84, 0x86, 0x88, 0x89, 0x8b, 0x8d, 0x8f, 0x91, 0x92, 0x94, 
0x96, 0x98, 0x99, 0x9b, 0x9d, 0x9f, 0xa0, 0xa2, 0xa4, 0xa5, 0xa7, 0xa9, 0xaa, 0xac, 0xae, 0xaf, 
0xb1, 0xb3, 0xb4, 0xb6, 0xb7, 0xb9, 0xba, 
};

static const uint16_t space_tab[] = {
0xfa, 0xfa, 0xfa, 0xfa, 0xf9, 0xf9, 0xf8, 0xf8, 0xf7, 0xf7, 0xf6, 0xf5, 0xf4, 0xf3, 0xf2, 0xf0, 
0xef, 0xee, 0xec, 0xeb, 0xe9, 0xe8, 0xe6, 0xe4, 0xe2, 0xe0, 0xde, 0xdc, 0xda, 0xd8, 0xd5, 0xd3, 
0xd1, 0xce, 0xcc, 0xc9, 0xc6, 0xc4, 0xc1, 0xbe, 0xbc, 0xb9, 0xb6, 0xb3, 0xb0, 0xad, 0xaa, 0xa7, 
0xa4, 0xa1, 0x9d, 0x9a, 0x97, 0x94, 0x91, 0x8d, 0x8a, 0x87, 0x84, 0x80, 0x7d, 0x7a, 0x76, 0x73, 
0x70, 0x6d, 0x69, 0x66, 0x63, 0x60, 0x5d, 0x59, 0x56, 0x53, 0x50, 0x4d, 0x4a, 0x47, 0x44, 0x41, 
0x3f, 0x3c, 0x39, 0x36, 0x34, 0x31, 0x2e, 0x2c, 0x29, 0x27, 0x25, 0x22, 0x20, 0x1e, 0x1c, 0x1a, 
0x18, 0x16, 0x14, 0x12, 0x11, 0x0f, 0x0e, 0x0c, 0x0b, 0x0a, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 
0x03, 0x02, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 
0x03, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x0a, 0x0b, 0x0c, 0x0e, 0x0f, 0x11, 0x12, 0x14, 0x16, 
0x18, 0x1a, 0x1c, 0x1e, 0x20, 0x22, 0x25, 0x27, 0x29, 0x2c, 0x2e, 0x31, 0x34, 0x36, 0x39, 0x3c, 
0x3e, 0x41, 0x44, 0x47, 0x4a, 0x4d, 0x50, 0x53, 0x56, 0x59, 0x5d, 0x60, 0x63, 0x66, 0x69, 0x6d, 
0x70, 0x73, 0x76, 0x7a, 0x7d, 0x80, 0x84, 0x87, 0x8a, 0x8d, 0x91, 0x94, 0x97, 0x9a, 0x9d, 0xa1, 
0xa4, 0xa7, 0xaa, 0xad, 0xb0, 0xb3, 0xb6, 0xb9, 0xbc, 0xbe, 0xc1, 0xc4, 0xc6, 0xc9, 0xcc, 0xce, 
0xd1, 0xd3, 0xd5, 0xd8, 0xda, 0xdc, 0xde, 0xe0, 0xe2, 0xe4, 0xe6, 0xe8, 0xe9, 0xeb, 0xec, 0xee, 
0xef, 0xf0, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf7, 0xf8, 0xf8, 0xf9, 0xf9, 0xfa, 0xfa, 0xfa, 
0xfa, 0xfa, 0xfa, 0xfa, 0xf9, 0xf9, 0xf8, 0xf8, 0xf7, 0xf7, 0xf6, 0xf5, 0xf4, 0xf3, 0xf2, 0xf0, 
0xef, 0xee, 0xec, 0xeb, 0xe9, 0xe8, 0xe6, 0xe4, 0xe2, 0xe0, 0xde, 0xdc, 0xda, 0xd8, 0xd5, 0xd3, 
0xd1, 0xce, 0xcc, 0xc9, 0xc6, 0xc4, 0xc1, 0xbe, 0xbc, 0xb9, 0xb6, 0xb3, 0xb0, 0xad, 0xaa, 0xa7, 
0xa4, 0xa1, 0x9d, 0x9a, 0x97, 0x94, 0x91, 0x8d, 0x8a, 0x87, 0x84, 0x80, 0x7d, 0x7a, 0x76, 0x73, 
0x70, 0x6d, 0x69, 0x66, 0x63, 0x60, 0x5d, 0x59, 0x56, 0x53, 0x50, 0x4d, 0x4a, 0x47, 0x44, 0x41, 
0x3f, 0x3c, 0x39, 0x36, 0x34, 0x31, 0x2e, 0x2c, 0x29, 0x27, 0x25, 0x22, 0x20, 0x1e, 0x1c, 0x1a, 
0x18, 0x16, 0x14, 0x12, 0x11, 0x0f, 0x0e, 0x0c, 0x0b, 0x0a, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 
0x03, 0x02, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 
0x03, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x0a, 0x0b, 0x0c, 0x0e, 0x0f, 0x11, 0x12, 0x14, 0x16, 
0x18, 0x1a, 0x1c, 0x1e, 0x20, 0x22, 0x25, 0x27, 0x29, 0x2c, 0x2e, 0x31, 0x34, 0x36, 0x39, 0x3c, 
0x3f, 0x41, 0x44, 0x47, 0x4a, 0x4d, 0x50, 0x53, 0x56, 0x59, 0x5d, 0x60, 0x63, 0x66, 0x69, 0x6d, 
0x70, 0x73, 0x76, 0x7a, 0x7d, 0x80, 0x84, 0x87, 0x8a, 0x8d, 0x91, 0x94, 0x97, 0x9a, 0x9d, 0xa1, 
0xa4, 0xa7, 0xaa, 0xad, 0xb0, 0xb3, 0xb6, 0xb9, 0xbc, 0xbe, 0xc1, 0xc4, 0xc6, 0xc9, 0xcc, 0xce, 
0xd1, 0xd3, 0xd5, 0xd8, 0xda, 0xdc, 0xde, 0xe0, 0xe2, 0xe4, 0xe6, 0xe8, 0xe9, 0xeb, 0xec, 0xee, 
0xef, 0xf0, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf7, 0xf8, 0xf8, 0xf9, 0xf9, 0xfa, 0xfa, 0xfa, 
0xfa, 0xfa, 0xfa, 0xfa, 0xf9, 0xf9, 0xf8, 0xf8, 0xf7, 0xf7, 0xf6, 0xf5, 0xf4, 0xf3, 0xf2, 0xf0, 
0xef, 0xee, 0xec, 0xeb, 0xe9, 0xe8, 0xe6, 0xe4, 0xe2, 0xe0, 0xde, 0xdc, 0xda, 0xd8, 0xd5, 0xd3, 
0xd1, 0xce, 0xcc, 0xc9, 0xc6, 0xc4, 0xc1, 0xbe, 0xbc, 0xb9, 0xb6, 0xb3, 0xb0, 0xad, 0xaa, 0xa7, 
0xa4, 0xa1, 0x9d, 0x9a, 0x97, 0x94, 0x91, 0x8d, 0x8a, 0x87, 0x84, 0x80, 0x7d, 0x7a, 0x76, 0x73, 
0x70, 0x6d, 0x69, 0x66, 0x63, 0x60, 0x5d, 0x59, 0x56, 0x53, 0x50, 0x4d, 0x4a, 0x47, 0x44, 0x41, 
0x3f, 0x3c, 0x39, 0x36, 0x34, 0x31, 0x2e, 0x2c, 0x29, 0x27, 0x25, 0x22, 0x20, 0x1e, 0x1c, 0x1a, 
0x18, 0x16, 0x14, 0x12, 0x11, 0x0f, 0x0e, 0x0c, 0x0b, 0x0a, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 
0x03, 0x02, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 
0x03, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x0a, 0x0b, 0x0c, 0x0e, 0x0f, 0x11, 0x12, 0x14, 0x16, 
0x18, 0x1a, 0x1c, 0x1e, 0x20, 0x22, 0x25, 0x27, 0x29, 0x2c, 0x2e, 0x31, 0x34, 0x36, 0x39, 0x3c, 
};

static const uint16_t *phase_tab[2][PHASE_CYCLE] = {
    { &space_tab[0], &space_tab[40], &space_tab[80], &space_tab[120], &space_tab[160], &space_tab[200], },
    { &mark_tab[0], &mark_tab[73], &mark_tab[147], &mark_tab[220], &mark_tab[293], &mark_tab[367], },
};
